package com.cultural.heritage.listener;

import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.constant.MqConstant;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.entity.UserCoupon;
import com.cultural.heritage.model.vo.coupon.CouponVO;
import com.cultural.heritage.service.good.UserCouponService;
import com.cultural.heritage.utils.MultiDelayMessage;
import com.google.gson.Gson;
import jakarta.annotation.Resource;
import org.springframework.amqp.rabbit.annotation.Exchange;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.QueueBinding;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.stereotype.Component;


@Component
public class UserCouponStatusListener {

    @Resource
    private UserCouponService userCouponService;


    @Resource
    private RabbitTemplate rabbitTemplate;


    @RabbitListener(bindings = @QueueBinding(
            value = @Queue(MqConstant.DELAY_USER_COUPON_QUEUE),
            exchange = @Exchange(name = MqConstant.DELAY_EXCHANGE, delayed = "true"),
            key = MqConstant.DELAY_USER_COUPON_ROUTING_KEY
    ))
    public void listenDelayMessage(MultiDelayMessage<Long> msg) {
        System.out.println("\n\n\n\n\nUserCouponStatusListener.listenerDelayMessage msg-------------------------------->：" + msg);

        // 3.我的优惠券未到截止日期，判断是否还有剩余延时时间
        if (msg.hasNextDelay()) {
            // 有延迟时间，需要重发延迟消息，先获取延迟时间的int值
            // 发送延时消息
            int delayValue = msg.removeNextDelay().intValue();
            rabbitTemplate.convertAndSend(MqConstant.DELAY_EXCHANGE,
                    MqConstant.DELAY_ORDER_ROUTING_KEY, msg, message -> {
                        // 添加延迟消息属性
                        message.getMessageProperties().setDelay(delayValue * 1000);
                        return message;
                    });
            return ;
        }

        // 1.获取用户优惠券id
        Long userCouponId = msg.getData();
        // 2.获取优惠券信息
        UserCoupon userCoupon = userCouponService.getById(userCouponId);
        CouponVO couponVO = userCoupon.getCouponVO();
        couponVO.setStatus("已过期");
        // 3.更新优惠券状态
        UpdateWrapper<UserCoupon> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", userCouponId).set("couponVO", new Gson().toJson(couponVO));
        boolean update = userCouponService.update(updateWrapper);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "用户优惠券状态更新失败");

    }


}
