package com.cultural.heritage.model.dto.coupon;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;

@Data
@Schema(description = "优惠券更新请求体", requiredProperties =
        {"id", "name", "conditionAmount", "requirePoints", "content",  "startTime", "endTime", "description"})
public class CouponUpdateRequest implements Serializable {

    /**
     * 优惠券id
     */
    @Schema(description = "优惠券id(id > 0)", example = "4")
    private Long id;

    /**
     * 优惠券名称
     */
    @Schema(description = "优惠券名称", example = "满200减50")
    private String name;

    /**
     * 标准金额
     */
    @Schema(description = "标准金额", example = "200")
    private BigDecimal standardAmount;

    /**
     * 满减金额
     */
    @Schema(description = "满减金额", example = "50")
    private BigDecimal conditionAmount;


    /**
     * 需要的积分
     */
    @Schema(description = "兑换需要的积分", example = "1000")
    private Integer requirePoints;


    /**
     * 有效截止日期
     */
    @Schema(description = "有效截止日期", example = "2024-12-04 15:00:00")
    private String endTime;


    /**
     * 使用说明
     */
    @Schema(description = "使用说明", example = "该优惠券只作用于一个材料包类商品;取消订单后优惠券会自动退回")
    private String description;



    @Serial
    private static final long serialVersionUID = 1L;
}
