package com.cultural.heritage.model.dto.good.service;

import com.cultural.heritage.model.dto.appointment.AppointmentDateAddRequest;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;

@Data
@Schema(description = "服务类商品添加请求体", requiredProperties = {"name", "price", "goodImg", "intro", "introDetail",
"detailImg", "label", "appointmentDateAddRequestList"})
public class ServiceGoodAddRequest implements Serializable {

    /**
     * 商品名
     */
    @Schema(description = "商品名", example = "非遗香囊")
    private String name;

    /**
     * 商品价格
     */
    @Schema(description = "商品价格", example = "20.00")
    private BigDecimal price;

    /**
     * 商品图片
     */
    @Schema(description = "商品图片", example = "https://xxx/xxx.jpg")
    private String goodImg;

    /**
     * 商品简介
     */
    @Schema(description = "商品简介", example = "传承千年文化，守护健康美好")
    private String intro;

    /**
     * 商品详情简介
     */
    @Schema(description = "商品详情简介", example = "精选药材：选用艾草、菖蒲、苍术、白芷等十多种纯天然中草药，科学配比，香气宜人，具有驱蚊、防疫、安神等多种功效。端午香囊，传承千年文化，守护健康美好。在这个端午节，让我们共同感受传统文化的魅力，为生活增添一抹色彩！")
    private String introDetail;

    /**
     * 商品详情图片
     */
    @Schema(description = "商品详情图片", example = "https://xxx/xxx.jpg")
    private String detailImg;

    /**
     * 商品标签
     */
    @Schema(description = "商品标签", example = "亲情;送礼;材料包")
    private String label;


    /**
     * 未来的预约详情
     */
    @Schema(description = "未来的预约详情")
    private List<AppointmentDateAddRequest> appointmentDateAddRequestList;



    @Serial
    private static final long serialVersionUID = 1L;
}
