package com.cultural.heritage.service.wx.impl;


import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.config.WxOpenConfig;
import com.cultural.heritage.config.WxPayConfig;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.snapshot.CouponSnapshot;
import com.cultural.heritage.model.dto.snapshot.GoodSnapshot;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.service.book.AdvanceOrderService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.service.order.PendingServiceOrderService;
import com.cultural.heritage.service.order.RefundRecordService;
import com.cultural.heritage.service.wx.WeChatService;
import com.cultural.heritage.utils.RefundUtils;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.jsapi.model.Amount;
import com.wechat.pay.java.service.payments.jsapi.model.Payer;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayRequest;
import com.wechat.pay.java.service.payments.jsapi.model.PrepayWithRequestPaymentResponse;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.AmountReq;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author 玄德
 */
@Slf4j
@Service
public class WeChatServiceImpl implements WeChatService {


    @Resource
    private WxPayConfig wxPayConfig;

    @Resource
    private WxOpenConfig wxOpenConfig;

    @Resource
    private OrderService orderService;

    @Resource
    private GoodService goodService;

    @Resource
    private OrderItemService orderItemService;

    @Resource
    private RefundRecordService refundRecordService;

    @Resource
    private PendingServiceOrderService pendingServiceOrderService;

    @Resource
    private AdvanceOrderService advanceOrderService;


    /**
     * 请求参数
     */
    public static RequestParam requestParam = null;


    /**
     * 微信支付(商品类）
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized PrepayWithRequestPaymentResponse createPayment(String orderId, String miniOpenId, BigDecimal amount) {
        // request.setXxx(val)设置所需参数，具体参数可见Request定义
        PrepayRequest request = new PrepayRequest();
        // 金额
        Amount WxAmount = new Amount();
        WxAmount.setTotal(amount.movePointRight(2).intValue());
        WxAmount.setCurrency("CNY");
        request.setAmount(WxAmount);
        // 公众号id
        request.setAppid(wxPayConfig.getAppId());
        // 商户号
        request.setMchid(wxPayConfig.getMerchantId());
        // 支付者信息
        Payer payer = new Payer();
        payer.setOpenid(miniOpenId);
        request.setPayer(payer);
        // 获取订单号
        Order order = orderService.getById(orderId);
        String orderNumber = order.getOrderNumber();
        // 描述
        request.setDescription("订单号：" + orderNumber);
        // 微信回调地址
        request.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/payment/callback");
        // 商户订单号
        request.setOutTradeNo(orderNumber);
        //返回数据，前端调起支付
        return wxPayConfig.getJsapiServiceExtension().prepayWithRequestPayment(request);
    }


    /**
     * 微信支付（写真预约类）
     */
    @Override
    public PrepayWithRequestPaymentResponse createPhotoProductsPayment(String orderId, String miniOpenId, BigDecimal amount) {
        // request.setXxx(val)设置所需参数，具体参数可见Request定义
        PrepayRequest request = new PrepayRequest();
        // 金额
        Amount WxAmount = new Amount();
        WxAmount.setTotal(amount.movePointRight(2).intValue());
        WxAmount.setCurrency("CNY");
        request.setAmount(WxAmount);
        // 公众号id
        request.setAppid(wxPayConfig.getAppId());
        // 商户号
        request.setMchid(wxPayConfig.getMerchantId());
        // 支付者信息
        Payer payer = new Payer();
        payer.setOpenid(miniOpenId);
        request.setPayer(payer);
        // 获取订单号
        AdvanceOrder advanceOrder = advanceOrderService.getById(orderId);
        String orderNumber = advanceOrder.getOrderNumber();
        // 描述
        request.setDescription("订单号：" + orderNumber);
        // 微信回调地址
        request.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/payment/photo/callback");
        // 商户订单号
        request.setOutTradeNo(orderNumber);
        //返回数据，前端调起支付
        return wxPayConfig.getJsapiServiceExtension().prepayWithRequestPayment(request);
    }


    /**
     * 支付回调（商品类）
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized boolean paymentCallback(Transaction transaction) {
        System.out.println("---------------------------微信支付回调（开始）-------------------------------");
        // 获取订单信息
        String orderNumber = transaction.getOutTradeNo();
        Order order = getOrderInfoByOrderNumber(orderNumber);

        // 修改订单状态
        modifyOrderStatus(order, OrderStatusConstant.PENDING_SHIPMENT);

        boolean isGeneral = orderService.isGeneralGood(order.getOrderType());

        if (!isGeneral) {
            // 更新服务类商品待处理订单记录
            updatePendingServiceOrder(orderNumber, OrderStatusConstant.PENDING_SHIPMENT);
        }

        System.out.println("---------------------------微信支付回调（结束）-------------------------------");
        return true;
    }


    /**
     * 支付回调（写真预约类）
     */
    @Override
    public boolean paymentPhotoProductsCallback(Transaction transaction) {
        System.out.println("---------------------------微信支付回调（开始）-------------------------------");
        // 获取订单信息
        String orderNumber = transaction.getOutTradeNo();
        AdvanceOrder advanceOrder = getAdvanceOrderInfoByOrderNumber(orderNumber);

        // 修改订单状态
        modifyAdvanceOrderStatus(advanceOrder, OrderStatusConstant.PENDING_SHIPMENT);
        System.out.println("---------------------------微信支付回调（结束）-------------------------------");
        return true;
    }



    /**
     * 全额退款（商品类）
     */
    @Override
    public Refund refundPayment(String orderId, BigDecimal amount) {
        // 获取订单号
        Order order = orderService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "该订单不存在");
        String orderNumber = order.getOrderNumber();
        // 退款请求
        CreateRequest createRequest = new CreateRequest();
        // 商户订单号
        createRequest.setOutTradeNo(orderNumber);
        // 商户退款单号
        String outRefundNo = RefundUtils.generateRefundNo();
        createRequest.setOutRefundNo(outRefundNo);
        // 退款结果回调
        createRequest.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/refund/callback");
        // 退款金额
        AmountReq amountReq = new AmountReq();
        long refundAmount = amount.movePointRight(2).intValue();

        amountReq.setRefund(refundAmount);
        amountReq.setTotal(refundAmount);
        amountReq.setCurrency("CNY");
        createRequest.setAmount(amountReq);
        // 申请退款
        System.out.println("退款请求：" + createRequest);
        Refund refund = wxPayConfig.getRefundService().create(createRequest);
        System.out.println("退款申请结果：" + refund);

        return refund;
    }



    /**
     * 全额退款（写真预约类）
     */
    @Override
    public Refund refundPhotoProductsPayment(String orderId, BigDecimal amount) {
        // 获取订单号
        AdvanceOrder advanceOrder = advanceOrderService.getById(orderId);
        ThrowUtils.throwIf(advanceOrder == null, ErrorCode.OPERATION_ERROR, "该订单不存在");
        String orderNumber = advanceOrder.getOrderNumber();
        // 退款请求
        CreateRequest createRequest = new CreateRequest();
        // 商户订单号
        createRequest.setOutTradeNo(orderNumber);
        // 商户退款单号
        String outRefundNo = RefundUtils.generateRefundNo();
        createRequest.setOutRefundNo(outRefundNo);
        // 退款结果回调
        createRequest.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/refund/photo/callback");
        // 退款金额
        AmountReq amountReq = new AmountReq();
        long refundAmount = amount.movePointRight(2).intValue();

        amountReq.setRefund(refundAmount);
        amountReq.setTotal(refundAmount);
        amountReq.setCurrency("CNY");
        createRequest.setAmount(amountReq);
        // 申请退款
        System.out.println("退款请求：" + createRequest);
        Refund refund = wxPayConfig.getRefundService().create(createRequest);
        System.out.println("退款申请结果：" + refund);
        return refund;
    }



    /**
     * 全额退款回调（商品类）
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized boolean refundCallback(RefundNotification refundNotification) {
        System.out.println("---------------------------微信退款回调（开始）-------------------------------");
        // 获取订单信息
        String orderNumber = refundNotification.getOutTradeNo();
        Order order = getOrderInfoByOrderNumber(orderNumber);

        // 修改订单状态
        modifyOrderStatus(order, OrderStatusConstant.PAYMENT_REFUNDED);

        // 生成退款记录
        createRefundRecord(refundNotification);

        boolean isGeneral = orderService.isGeneralGood(order.getOrderType());
        if (!isGeneral) {
            // 更新服务类订单待处理记录
            updatePendingServiceOrder(order.getOrderNumber(), OrderStatusConstant.PAYMENT_REFUNDED);
        } else {
            // 恢复商品库存
            recoverGoodInventory(order);
        }
        System.out.println("---------------------------微信退款回调（结束）-------------------------------");
        return true;
    }




    /**
     * 全额退款回调（写真预约类）
     */
    @Override
    public boolean refundPhotoProductsCallback(RefundNotification refundNotification) {
        System.out.println("---------------------------微信退款回调（开始）-------------------------------");
        // 获取订单信息
        String orderNumber = refundNotification.getOutTradeNo();
        AdvanceOrder advanceOrder = getAdvanceOrderInfoByOrderNumber(orderNumber);

        // 修改订单状态
        modifyAdvanceOrderStatus(advanceOrder, OrderStatusConstant.PAYMENT_REFUNDED);

        // 生成退款记录
        createRefundRecord(refundNotification);
        System.out.println("---------------------------微信退款回调（结束）-------------------------------");
        return true;
    }




    /**
     * 部分退款
     */
    @Override
    public Refund refundPartPayment(String orderItemsId, BigDecimal amount) {
        // 获取订单明细
        OrderItems orderItems = orderItemService.getById(orderItemsId);
        ThrowUtils.throwIf(orderItems == null, ErrorCode.OPERATION_ERROR, "订单明细不存在");
        // 获取订单明细所属的订单
        Order order = orderService.getById(orderItems.getOrderId());
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        // 判断该订单明细是否已经退款
        checkOrderItemStatus(orderItemsId, order);

        String orderNumber = order.getOrderNumber();
        // 退款请求
        CreateRequest createRequest = new CreateRequest();
        // 商户订单号
        createRequest.setOutTradeNo(orderNumber);
        // 商户退款单号
        String outRefundNo = RefundUtils.generateRefundPartNo(orderItemsId);
        createRequest.setOutRefundNo(outRefundNo);
        // 退款结果回调
        createRequest.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/refund/part/callback");
        // 退款金额
        AmountReq amountReq = new AmountReq();

        // 计算部分退款金额
        BigDecimal refundAmount = getPartRefundAmount(order, amount);

        amountReq.setRefund(refundAmount.longValue());
        amountReq.setTotal(order.getTotalAmount().movePointRight(2).longValue());
        amountReq.setCurrency("CNY");
        createRequest.setAmount(amountReq);

        // 生成退款记录
        createRefundRecord(orderNumber, outRefundNo, refundAmount);

        // 申请退款
        System.out.println("退款请求：" + createRequest);
        Refund refund = wxPayConfig.getRefundService().create(createRequest);
        System.out.println("退款申请结果：" + refund);

        return refund;
    }






    /**
     * 部分退款回调
     */
    @Override
    public boolean refundPartCallback(RefundNotification refundNotification) {
        System.out.println("---------------------------微信退款回调（开始）-------------------------------");
        // 获取订单信息
        String orderNumber = refundNotification.getOutTradeNo();
        Order order = getOrderInfoByOrderNumber(orderNumber);

        // 解析出订单明细id
        String outRefundNo = refundNotification.getOutRefundNo();
        String orderItemId = RefundUtils.parseRefundNoToItemId(outRefundNo);

        boolean isGeneral = orderService.isGeneralGood(order.getOrderType());
        if (!isGeneral) {
            // 更新服务类订单待处理记录
            UpdateWrapper<PendingServiceOrder> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("orderItemId", orderItemId).set("orderItemStatus", OrderStatusConstant.PAYMENT_REFUNDED);
            boolean update = pendingServiceOrderService.update(updateWrapper);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "服务类订单待处理记录状态更新失败");
        } else {
            // 恢复商品库存
            OrderItems orderItems = orderItemService.getById(orderItemId);
            Long id = orderItems.getGoodSnapshot().getId();
            Good good = goodService.getById(id);
            ThrowUtils.throwIf(good == null, ErrorCode.OPERATION_ERROR, "当前商品不存在");
            good.setInventory(good.getInventory() + orderItems.getQuantity());
            boolean update = goodService.updateById(good);
            ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "商品库存恢复失败");
        }
        System.out.println("---------------------------微信退款回调（结束）-------------------------------");
        return true;
    }




    /**
     * 剩余退款申请
     */
    @Override
    public Refund refundRestPayment(String orderId, BigDecimal totalAmount) {
        // 获取订单号
        Order order = orderService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "该订单不存在");
        String orderNumber = order.getOrderNumber();
        // 退款请求
        CreateRequest createRequest = new CreateRequest();
        // 商户订单号
        createRequest.setOutTradeNo(orderNumber);
        // 商户退款单号
        String outRefundNo = RefundUtils.generateRefundNo();
        createRequest.setOutRefundNo(outRefundNo);
        // 退款结果回调
        createRequest.setNotifyUrl(wxPayConfig.getNotifyUrl() + "/api/wechat/refund/rest/callback");
        // 退款金额
        AmountReq amountReq = new AmountReq();
        long refundAmount = totalAmount.movePointRight(2).intValue();

        // 获取剩余退款的金额
        BigDecimal restRefundAmount = this.getRestRefundAmount(order);

        amountReq.setRefund(restRefundAmount.movePointRight(2).longValue());
        amountReq.setTotal(refundAmount);
        amountReq.setCurrency("CNY");
        createRequest.setAmount(amountReq);
        // 申请退款
        System.out.println("退款请求：" + createRequest);
        Refund refund = wxPayConfig.getRefundService().create(createRequest);
        System.out.println("退款申请结果：" + refund);

        return refund;
    }



    /**
     * 剩余退款回调
     */
    @Override
    public boolean refundRestCallback(RefundNotification refundNotification) {

        System.out.println("---------------------------微信退款回调（开始）-------------------------------");
        // 获取订单信息
        String orderNumber = refundNotification.getOutTradeNo();
        Order order = getOrderInfoByOrderNumber(orderNumber);

        // 修改订单状态
        modifyOrderStatus(order, OrderStatusConstant.PAYMENT_REFUNDED);

        // 生成退款记录
        createRefundRecord(refundNotification);

        boolean isGeneral = orderService.isGeneralGood(order.getOrderType());
        if (!isGeneral) {
            // 更新服务类订单待处理记录
            updatePendingServiceOrder(order.getOrderNumber(), OrderStatusConstant.PAYMENT_REFUNDED);
        } else {
            // 恢复商品库存
            recoverGoodInventory(order);
        }
        System.out.println("---------------------------微信退款回调（结束）-------------------------------");
        return true;
    }























    /**
     * 获取剩余退款的金额
     */
    private BigDecimal getRestRefundAmount(Order order) {
        QueryWrapper<RefundRecord> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("outTradeNo", order.getOrderNumber());
        List<RefundRecord> refundRecordList = refundRecordService.list(queryWrapper);
        BigDecimal totalRefund = new BigDecimal("0.00");
        for (RefundRecord refundRecord : refundRecordList) {
            totalRefund = totalRefund.add(refundRecord.getRefundAmount());
        }
        return order.getTotalAmount().subtract(totalRefund);
    }



    /**
     * 判断当前订单明细是否已退款
     */
    private void checkOrderItemStatus(String orderItemsId, Order order) {
        boolean isGeneral = orderService.isGeneralGood(order.getOrderType());
        if (isGeneral) {
            QueryWrapper<RefundRecord> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("outTradeNo", order.getOrderNumber());
            List<RefundRecord> refundRecordList = refundRecordService.list(queryWrapper);
            for (RefundRecord refundRecord : refundRecordList) {
                String itemId = RefundUtils.parseRefundNoToItemId(refundRecord.getOutRefundNo());
                System.out.println("订单明细id----------------------------------------------------------------------------------------------------->" + itemId);
                if (orderItemsId.equals(itemId)) throw new BusinessException(ErrorCode.PARAMS_ERROR, "当前订单明细已退款");
            }
        } else {
            QueryWrapper<PendingServiceOrder> pendingServiceOrderQueryWrapper = new QueryWrapper<>();
            pendingServiceOrderQueryWrapper.eq("orderItemId", orderItemsId);
            PendingServiceOrder pendingServiceOrder = pendingServiceOrderService.getOne(pendingServiceOrderQueryWrapper);
            ThrowUtils.throwIf(pendingServiceOrder == null || !pendingServiceOrder.getOrderItemStatus()
                    .equals(OrderStatusConstant.PENDING_SHIPMENT), ErrorCode.OPERATION_ERROR, "服务类订单待处理记录不存在或者当前订单明细状态错误");
        }

    }



    /**
     * 计算部分退款金额
     */
    private BigDecimal getPartRefundAmount(Order order, BigDecimal amount) {

        BigDecimal refundAmount;
        CouponSnapshot couponSnapshot = order.getCouponSnapshot();
        if (couponSnapshot == null) {
            refundAmount = amount;
        } else {
            // 判断是否为最后一个商品，如果是，则将剩余金额全部退款
            QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("orderId", order.getId());
            long itemCount = orderItemService.count(queryWrapper);

            QueryWrapper<RefundRecord> recordQueryWrapper = new QueryWrapper<>();
            recordQueryWrapper.eq("outTradeNo", order.getOrderNumber());
            long refundCount = refundRecordService.count(recordQueryWrapper);

            if (itemCount - refundCount == 1) {
                List<RefundRecord> refundRecords = refundRecordService.list(recordQueryWrapper);
                BigDecimal refundTotalAmount = new BigDecimal("0.00");
                for (RefundRecord refundRecord : refundRecords) {
                    BigDecimal recordRefundAmount = refundRecord.getRefundAmount();
                    refundTotalAmount = refundTotalAmount.add(recordRefundAmount);
                }
                refundAmount = order.getTotalAmount().subtract(refundTotalAmount);
            } else {
                BigDecimal couponAmount = couponSnapshot.getConditionAmount();
                BigDecimal totalAmount = order.getTotalAmount().add(couponAmount);
                BigDecimal itemShare = amount.divide(totalAmount, 10, RoundingMode.HALF_UP);
                BigDecimal itemCouponRefund = itemShare.multiply(couponAmount);
                refundAmount = amount.subtract(itemCouponRefund);
            }
        }
        refundAmount = refundAmount.movePointRight(2).setScale(0, RoundingMode.HALF_UP);
        return refundAmount;
    }




    /**
     * 更新服务类商品订单待处理记录
     */
    private void updatePendingServiceOrder(String orderNumber, String orderStatus) {
        QueryWrapper<PendingServiceOrder> orderQueryWrapper = new QueryWrapper<>();
        orderQueryWrapper.eq("orderNumber", orderNumber);
        List<PendingServiceOrder> pendingServiceOrderList = pendingServiceOrderService.list(orderQueryWrapper);
        for (PendingServiceOrder pendingServiceOrder : pendingServiceOrderList) {
            pendingServiceOrder.setOrderItemStatus(orderStatus);
        }
        boolean result = pendingServiceOrderService.updateBatchById(pendingServiceOrderList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "服务类订单待处理记录批量更新失败");
    }






    /**
     * 根据订单号获取订单信息
     */
    private Order getOrderInfoByOrderNumber(String orderNumber) {
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderNumber", orderNumber);
        Order order = orderService.getOne(queryWrapper);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        return order;
    }


    /**
     * 根据订单号获取写真预约类信息
     */
    private AdvanceOrder getAdvanceOrderInfoByOrderNumber(String orderNumber) {
        QueryWrapper<AdvanceOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderNumber", orderNumber);
        AdvanceOrder advanceOrder = advanceOrderService.getOne(queryWrapper);
        ThrowUtils.throwIf(advanceOrder == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        return advanceOrder;
    }




    /**
     * 生成退款记录
     */
    private void createRefundRecord(RefundNotification refundNotification) {
        RefundRecord refundRecord = new RefundRecord();
        refundRecord.setOutTradeNo(refundNotification.getOutTradeNo());
        refundRecord.setOutRefundNo(refundNotification.getOutRefundNo());
        refundRecord.setRefundAmount(BigDecimal.valueOf(refundNotification.getAmount().getRefund()).movePointLeft(2));

        boolean save = refundRecordService.save(refundRecord);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "退款记录生成失败");
    }



    /**
     * 生成退款记录
     */
    private void createRefundRecord(String outTradeNo, String outRefundNo, BigDecimal refundAmount) {
        RefundRecord refundRecord = new RefundRecord();
        refundRecord.setOutTradeNo(outTradeNo);
        refundRecord.setOutRefundNo(outRefundNo);
        refundRecord.setRefundAmount(refundAmount.movePointLeft(2));

        boolean save = refundRecordService.save(refundRecord);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR, "退款记录生成失败");
    }





    /**
     * 修改订单状态
     */
    private void modifyOrderStatus(Order order, String orderStatus) {
        order.setOrderStatus(orderStatus);
        order.setUpdateTime(DateUtil.date());
        boolean update = orderService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态修改失败");
    }


    /**
     * 修改写真预约订单状态
     */
    private void modifyAdvanceOrderStatus(AdvanceOrder advanceOrder, String orderStatus) {
        advanceOrder.setOrderStatus(orderStatus);
        advanceOrder.setUpdateTime(DateUtil.date());
        boolean update = advanceOrderService.updateById(advanceOrder);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态修改失败");
    }



    /**
     * 恢复商品库存
     */
    private void recoverGoodInventory(Order order) {
        // 获取订单明细id列表
        QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("orderId", order.getId());
        List<OrderItems> orderItemsList = orderItemService.list(queryWrapper);

        // 获取退款的订单明细id
        QueryWrapper<RefundRecord> recordQueryWrapper = new QueryWrapper<>();
        recordQueryWrapper.eq("outTradeNo", order.getOrderNumber());
        List<RefundRecord> refundRecordList = refundRecordService.list(recordQueryWrapper);
        List<Long> itemRefundIds = refundRecordList.stream().map(refundRecord ->
                Long.parseLong(RefundUtils.parseRefundNoToItemId(refundRecord.getOutRefundNo()))).toList();

        orderItemsList = orderItemsList.stream().filter(orderItems -> !itemRefundIds.contains(orderItems.getId())).toList();

        // 获取商品id列表
        List<Long> goodIds = orderItemsList.stream().map(orderItems -> {
            GoodSnapshot goodSnapshot = orderItems.getGoodSnapshot();
            return goodSnapshot.getId();
        }).toList();
        QueryWrapper<Good> goodQueryWrapper = new QueryWrapper<>();
        goodQueryWrapper.in("id", goodIds);
        // 获取商品列表
        List<Good> goodList = goodService.list(goodQueryWrapper);
        // 封装为map集合（键：商品id, 值：库存量）,用来存储购买的商品id对应的购买数量
        Map<Long, Integer> map = new HashMap<>();
        for (OrderItems orderItems : orderItemsList) {
            GoodSnapshot goodSnapshot = orderItems.getGoodSnapshot();
            Long goodId = goodSnapshot.getId();
            Integer quantity = orderItems.getQuantity();
            map.put(goodId, quantity);
        }

        // 批量更新商品库存
        for (Good good : goodList) {
            Long goodId = good.getId();
            // 已经下单购买的数量
            Integer quantity = map.get(goodId);
            // 原有的库存
            Integer inventory = good.getInventory();
            good.setInventory(quantity + inventory);
            good.setUpdateTime(DateUtil.date());
        }
        boolean updateBatch = goodService.updateBatchById(goodList);
        ThrowUtils.throwIf(!updateBatch, ErrorCode.SYSTEM_ERROR, "商品库存恢复失败");
    }






    /**
     * 获取支付回调信息
     */
    @Override
    public Transaction getTransactionInfo(HttpServletRequest request) {
        NotificationParser notificationParser = getNotificationParser(request);
        return notificationParser.parse(requestParam, Transaction.class);
    }



    /**
     * 获取退款回调信息
     */
    @Override
    public RefundNotification getRefundInfo(HttpServletRequest request) {
        NotificationParser notificationParser = getNotificationParser(request);
        return notificationParser.parse(requestParam, RefundNotification.class);
    }

//
//    @Override
//    public boolean sendSubscribeMessage(String openid, String templateId) {
//        try {
//            // 获取小程序对象
//            WxMaService wxMaService = wxOpenConfig.getWxMaService();
//            // 获取消息接口
//            WxMaMsgService msgService = wxMaService.getMsgService();
//            // 构建订阅消息体
//            WxMaSubscribeMessage wxMaSubscribeMessage = new WxMaSubscribeMessage();
//            wxMaSubscribeMessage.setToUser(openid);
//            wxMaSubscribeMessage.setTemplateId(templateId);
//            List<WxMaSubscribeMessage.MsgData> msgDataList = getMsgData();
//            wxMaSubscribeMessage.setData(msgDataList);
//            // 发送消息
//            msgService.sendSubscribeMsg(wxMaSubscribeMessage);
//            return true;
//        } catch (Exception e) {
//            log.error("发送订阅消息失败：", e);
//            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "发送订阅消息失败");
//        }
//    }

//    /**
//     * 提取消息数据
//     */
//    private static List<WxMaSubscribeMessage.MsgData> getMsgData() {
//        List<WxMaSubscribeMessage.MsgData> msgDataList = new ArrayList<>();
//        String[][] dataItems = {
//                {"character_string1", "2022060215050983711"},
//                {"time2", "2022-06-22 15:48:05"},
//                {"thing3", "测试店"},
//                {"amount4", "99999元"},
//                {"thing5", "收款到账，请查看"}
//        };
//        for (String[] item : dataItems) {
//            WxMaSubscribeMessage.MsgData msgData = new WxMaSubscribeMessage.MsgData();
//            msgData.setName(item[0]);
//            msgData.setValue(item[1]);
//            msgDataList.add(msgData);
//        }
//        return msgDataList;
//    }

    /**
     * 根据微信官方发送的请求获取信息
     */
    @SneakyThrows
    public NotificationParser getNotificationParser(HttpServletRequest request) {
        System.out.println("---------------------------获取信息-------------------------------");
        // 获取RSA配置
        NotificationParser notificationParser = new NotificationParser(wxPayConfig.getRSAConfig());
        // 构建请求
        StringBuilder bodyBuilder = new StringBuilder();
        BufferedReader reader = request.getReader();
        String line;
        while ((line = reader.readLine()) != null) {
            bodyBuilder.append(line);
        }
        String body = bodyBuilder.toString();
        String timestamp = request.getHeader("Wechatpay-Timestamp");
        String nonce = request.getHeader("Wechatpay-Nonce");
        String signature = request.getHeader("Wechatpay-Signature");
        String singType = request.getHeader("Wechatpay-Signature-Type");
        String wechatPayCertificateSerialNumber = request.getHeader("Wechatpay-Serial");
        requestParam = new RequestParam.Builder()
                .serialNumber(wechatPayCertificateSerialNumber)
                .nonce(nonce)
                .signature(signature)
                .timestamp(timestamp)
                .signType(singType)
                .body(body)
                .build();
        System.out.println(requestParam.toString());
        System.out.println("---------------------------信息获取完毕-------------------------------");
        return notificationParser;
    }
}
