package com.cultural.heritage.redis;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import redis.clients.jedis.Jedis;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class JedisTest {

    private Jedis jedis;

    /**
     * 初始化Jedis实例
     */
    @Before
    public void init() {
        //指定Redis服务器的IP地址和端口号
        jedis = new Jedis("127.0.0.1", 6379);
        System.out.println("连接成功");
    }

    @Test
    public void redisJedisTest() {
        //操作字符串
        //方法名与操作原生redis的命令一致
        jedis.set("dec", "hello");
        String dec = jedis.get("dec");
        System.out.println(dec);

        //操作list
        jedis.lpush("data", "v1", "v2", "v3");
        jedis.rpush("data", "v4", "v5", "v6");
        List<String> data = jedis.lrange("data", 0, -1);
        for (String s : data) {
            System.out.print(s + " ");
        }
        System.out.println();

        //操作hash
        HashMap<String, String> hashMap = new HashMap<>();
        hashMap.put("id", "1");
        hashMap.put("name", "huhai");
        hashMap.put("age", "24");
        jedis.hset("object", hashMap);
        Map<String, String> object = jedis.hgetAll("object");
        object.forEach((key, value) -> System.out.print(key + "=" + value + " "));
    }

    /**
     * 关闭Jedis连接
     */
    @After
    public void close() {
        jedis.close();
        System.out.println("连接已关闭");
    }
}

